@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2014, 2018, 2020, 2021, 2022, 2023, 2025 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2014, 2018, 2020, 2021, 2022, 2023, 2025, 2026 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2017, 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2017, 2018, 2020 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;; Copyright © 2018 Filip Toman <filip.toman@verum.com>
@@ -28,6 +28,7 @@
 
 (define-module (dzn ast lookup)
   #:use-module (srfi srfi-1)
+  #:use-module (srfi srfi-2)
   #:use-module (srfi srfi-26)
   #:use-module (srfi srfi-71)
 
@@ -453,6 +454,9 @@ null) and return its CONTEXT."
                       (ast:lookup-variable (ast:parent o <behavior>) port+name))
                     (lookup-shared-variable port name)))))))
 
+(define-method (.type (o <function>))
+  (.type (.signature o)))
+
 (define-method (.type (o <argument>))
   (ast:lookup o (.type.name o)))
 
@@ -510,12 +514,10 @@ null) and return its CONTEXT."
     (ast:lookup ((compose .behavior .type .port) o) type-name)))
 
 (define-method (.type (o <enum-literal>))
-  (let* ((parent (or (ast:parent o <shared-field-test>)
-                     (ast:parent o <shared-variable>)))
-         (type-name (.type.name o)))
-    (cond (parent
-           (let ((type-name (last (.ids type-name))))
-             (ast:lookup ((compose .behavior .type .port) parent) type-name)))
-          (else
-           (or (ast:parent o <enum>)
-               (ast:lookup o type-name))))))
+  (let ((type-name (.type.name o)))
+    (or (and-let* ((parent (or (ast:parent o <shared-field-test>)
+                               (ast:parent o <shared-variable>)))
+                   (type-name (last (.ids type-name))))
+          (ast:lookup ((compose .behavior .type .port) parent) type-name))
+        (ast:parent o <enum>)
+        (ast:lookup o type-name))))
