@@ -1,9 +1,9 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2017, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2014, 2017, 2020, 2021, 2022, 2023, 2024 Rutger van Beusekom <rutger@dezyne.org>
-;;; Copyright © 2020, 2021, 2022, 2023 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2014, 2017, 2020, 2021, 2022, 2023, 2024, 2025 Rutger (regtur) van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2020, 2021, 2022, 2023, 2024 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -28,7 +28,7 @@
   #:use-module (ice-9 match)
 
   #:use-module (dzn ast equal)
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast lookup)
   #:use-module (dzn ast)
   #:use-module (dzn code)
@@ -78,6 +78,9 @@
 (define (wfc-warning o message)
   (make <warning> #:ast o #:message message))
 
+(define (wfc-error->info o)
+  (wfc-info (.ast o) (.message o)))
+
 (define-method (wfc (o <model>))
   '())
 
@@ -283,7 +286,7 @@
    (let ((type (ast:type o)))
      (cond ((not type)
             `(,(wfc-error o (format #f "unknown type name `~a'" (type-name (.type.name o))))))
-           ((is-a? type <extern>)
+           ((and (not (feature-supported? "reply-data")) (is-a? type <extern>))
             `(,(wfc-error o (format #f "cannot use extern type `~a' as return type" (type-name (.type.name o))))))
            (else '())))
    (append-map wfc (ast:formal* o))
@@ -313,7 +316,7 @@
      (append-map wfc arguments)
      (cond
       ((and (ast:parent o <component>)
-            (ast:parent o <action>))
+            (as (.parent o) <action>))
        =>
        (lambda (ast)
          (let* ((count (length arguments))
@@ -326,7 +329,7 @@
                       `(,(wfc-error ast (format #f "argument count mismatch, expected ~a, found: ~a" event-count count))
                         ,(wfc-info event (format #f "for formals of event `~a' defined here" (.name event)))))
                   (append-map (argument-type-check o) arguments formals)))))))
-      ((let ((ast (ast:parent o <call>)))
+      ((let ((ast (as (.parent o) <call>)))
          (and ast
               (.function ast)
               ast))
@@ -475,13 +478,46 @@
            (.name o))))
       '()))
 
-(define-method (wfc (o <function>))
+(define-method (wfc-function (o <function>))
   (append
+   (duplicate o)
    (re-definition o)
    (wfc (.signature o))
    (wfc (.statement o))
    (missing-return o)))
 
+(define-method (duplicate (o <function>))
+  (let ((name (.name o)))
+    (if (not (is-a? name <duplicate>)) '()
+        `(,(wfc-error o (format #f "duplicate identifier `~a'" (.name name)))))))
+
+(define-method (wfc (o <function>))
+  (wfc-function o))
+
+(define-method (wfc (o <expression-function>))
+  (let* ((bool? (is-a? (ast:type o) <bool>))
+         (args? (pair? (ast:formal* o)))
+         (called? (.called? o))
+         (recursive? (.recursive? o))
+         (predicate-errors (wfc-predicate (.expression o)))
+         (name (.name o)))
+    (append
+     (wfc-function o)
+     (cond
+      (recursive?
+       `(,(wfc-error o (format #f "expression function `~a' is recursive" name))))
+      ((not bool?)
+       `(,(wfc-error o (format #f "expression function `~a' does not have bool as return type" name))))
+      (args?
+       `(,(wfc-error o (format #f "expression function `~a' has arguments" name))))
+      ((pair? predicate-errors)
+       `(,(wfc-error o (format #f "expression of expression function `~a' is not a predicate" name))
+         ,@(map wfc-error->info predicate-errors)))
+      (else
+       '()))
+     (if called? '()
+         `(,(wfc-warning o (format #f "expression function `~a' is never called" name)))))))
+
 
 ;;;
 ;;; Statements
@@ -524,8 +560,10 @@
                  '()))))))
   (let* ((expression (.expression o))
          (otherwise? (is-a? expression <otherwise>))
+         (predicate-errors (wfc-predicate expression))
          (wfce (wfc expression)))
-    (append wfce
+    (append predicate-errors
+            wfce
             (if (or otherwise? (pair? wfce)) '()
                 (typed-expression expression <bool>))
             (if otherwise? (otherwise o) '())
@@ -534,6 +572,27 @@
             (if (is-a? (.statement o) <statement>) (wfc (.statement o))
                 `(,(wfc-error o "statement expected"))))))
 
+(define-method (wfc-predicate (o <expression>))
+  '())
+
+(define-method (wfc-predicate (o <action>))
+  `(,(wfc-error o "action not allowed here")))
+
+(define-method  (wfc-predicate (o <call>))
+  (let* ((function (.function o))
+         (allowed? (not (and function
+                             (not (is-a? function <expression-function>))))))
+    (if allowed? '()
+        (let ((name (.name function)))
+          `(,(wfc-error o (format #f "call to function `~a' not allowed here" name)))))))
+
+(define-method (wfc-predicate (o <not>))
+  (wfc-predicate (.expression o)))
+
+(define-method (wfc-predicate (o <binary>))
+  (append (wfc-predicate (.left o))
+          (wfc-predicate (.right o))))
+
 (define-method (wfc (o <declarative-illegal>))
   ;; TODO; in source??
   '())
@@ -556,33 +615,35 @@
    (blocking o)
    (wfc (.statement o))))
 
-(define (wfc-defer-argument var)
-  (if (is-a? var <undefined>)
-      `(,(wfc-error var (format #f "undefined identifier `~a'" (.name var))))
-      (let* ((variable (.variable var))
-             (name (.variable.name var))
-             (type (and=> variable .type)))
-        (cond
-         ((is-a? var <shared>)
-          `(,(wfc-error
-              var
-              (format #f "cannot use interface variable `~a' as defer argument"
-                      name))
-            ,(wfc-info variable (format #f "variable `~a' defined here" name))))
-         ((not (ast:member? variable))
-          `(,(wfc-error var (format
-                             #f
-                             "cannot use local variable `~a' as defer argument"
-                             name))
-            ,(wfc-info var (format #f "variable `~a' defined here" name))))
-         ((is-a? type <extern>)
-          `(,(wfc-error var (format
-                             #f
-                             "cannot use data variable `~a' as defer argument"
-                             name))
-            ,(wfc-info var (format #f "variable `~a' defined here" name))))
-         (else
-          '())))))
+(define-method (wfc-defer-argument (o <var>))
+  (let* ((variable (.variable o))
+         (name (.variable.name o))
+         (type (and=> variable .type)))
+    (cond
+     ((not (ast:member? variable))
+      `(,(wfc-error
+          o
+          (format #f "cannot use local variable `~a' as defer argument" name))
+        ,(wfc-info o (format #f "variable `~a' defined here" name))))
+     ((is-a? type <extern>)
+      `(,(wfc-error
+          o
+          (format #f "cannot use data variable `~a' as defer argument" name))
+        ,(wfc-info o (format #f "variable `~a' defined here" name))))
+     (else
+      '()))))
+
+(define-method (wfc-defer-argument (o <shared-var>))
+  (let ((variable (.variable o))
+        (name (.variable.name o)))
+    `(,(wfc-error
+        o
+        (format #f "cannot use interface variable `~a' as defer argument" name))
+      ,(wfc-info variable (format #f "variable `~a' defined here" name)))))
+
+(define-method (wfc-defer-argument (o <undefined>))
+  (let ((name (.name o)))
+    `(,(wfc-error o (format #f "undefined identifier `~a'" name)))))
 
 (define-method (wfc (o <defer>))
   (let ((arguments (ast:argument* o)))
@@ -686,6 +747,12 @@
    (imperative-context o)
    (illegal o)))
 
+(define-method (wfc (o <invariant>))
+  (let* ((expression (.expression o))
+         (wfce (wfc expression))
+         (predicate-errors (wfc-predicate expression)))
+    (append wfce predicate-errors)))
+
 (define-method (wfc (o <reply>))
   (append
    (imperative-context o)
@@ -741,6 +808,8 @@
                (matching-types (filter (cut ast:equal? <> reply-type) types)))
           (cond
            ((pair? matching-types) '())
+           ((equal? reply-type-name "<unknown type>")
+            `(,(wfc-error o (format #f "cannot determine type of expression of reply"))))
            (port
             `(,(wfc-error o (format #f "type mismatch: no event with reply type `~a' for port `~a'"
                                     reply-type-name (.name port)))
@@ -1001,7 +1070,7 @@
          (called (.function o))
          (non-recursive? (or (not function) (not (.recursive? function))
                              (not called) (not (.recursive? called)))))
-    (if non-recursive? '()
+    (if (or non-recursive? (is-a? function <expression-function>)) '()
         (let* ((assign (or (ast:parent o <assign>)
                            (ast:parent o <return>)
                            (ast:parent o <variable>)))
@@ -1012,8 +1081,7 @@
                                   continuation)))
           (cond
            (assign
-            `(,(wfc-error o "cannot use typed function in recursion")
-              ,(wfc-info assign "statement after call")))
+            `(,(wfc-error o "cannot use typed function in recursion")))
            (continuation
             `(,(wfc-error o "cannot use statement after recursive call")
               ,(wfc-info continuation "statement after call")))
@@ -1046,7 +1114,6 @@
   (let ((name (ast:name o)))
     (ast:lookup (ast:parent o <scope>) name)))
 
-(use-modules (dzn ast accessor))
 (define (previous-definition-unshadowed o)
   "Disallow shadowing altogether"
   (let ((name (make <scope.name> #:ids (ast:full-name o))))
@@ -1404,7 +1471,9 @@
         `(,(wfc-error o (format #f "~a in member variable initializer" class)))))
      ((and (not (ast:parent o <on>))
            (not (ast:parent o <function>))
-           (not (and (equal? class "compound") (ast:declarative? o))))
+           (not (ast:parent o <invariant>))
+           (not (or (and (equal? class "compound") (ast:declarative? o))
+                    (and (is-a? o <call>) (ast:declarative? o)))))
       (let ((class (if (equal? class "compound") "imperative compound" class)))
         `(,(wfc-error o (format #f "~a outside on" class)))))
      ((and (is-a? (ast:type o) <void>)
@@ -1412,7 +1481,6 @@
       `(,(wfc-error o "void value not ignored as it ought to be")))
      ((and (not (is-a? (ast:type o) <void>))
            (or (is-a? p <compound>)
-               (is-a? p <guard>)
                (is-a? p <on>)
                (and (is-a? p <if>)
                     (not (ast:eq? o (.expression p))))))
