@@ -1,10 +1,10 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2020, 2021, 2022, 2023, 2024, 2025 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2020, 2021, 2022, 2023, 2024, 2025 Rutger (regtur) van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2016, 2017, 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018 Filip Toman <filip.toman@verum.com>
-;;; Copyright © 2023 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2022, 2023, 2024, 2025 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -34,11 +34,12 @@
   #:use-module (ice-9 string-fun)
 
   #:use-module (dzn ast display)
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast lookup)
   #:use-module (dzn ast normalize)
   #:use-module (dzn ast)
-  #:use-module (dzn code goops)
+  #:use-module (dzn code ast)
+  #:use-module (dzn code shared)
   #:use-module (dzn code language makreel)
   #:use-module (dzn command-line)
   #:use-module (dzn lts)
@@ -62,10 +63,7 @@
             code
 
             code:add-calling-context
-            code:add-calling-context-argument
-            code:add-calling-context-formal
             code:annotate-shells
-            code:argument*
             code:blocking-requires-in-void-returns
             code:blocking-return-values
             code:blocking?
@@ -82,7 +80,6 @@
             code:event-slot-name
             code:file-name
             code:file-name->string
-            code:formal*
             code:injected-binding*
             code:injected-instance*
             code:instance*
@@ -92,7 +89,6 @@
             code:member-name
             code:model*
             code:normalize
-            code:normalize+determinism
             code:number-argument
             code:number-formals
             code:on
@@ -169,8 +165,7 @@
                           (disjoin (is? <type>) (is? <namespace>)
                                    ast:imported?))
                          models))
-         (models (ast:topological-model-sort models))
-         (models (map code:annotate-shells models)))
+         (models (ast:topological-model-sort models)))
     models))
 
 (define-method (code:interface-include* (o <top>) source-file)
@@ -378,6 +373,9 @@
                  (%type-prefix ""))
     (code:type-name o)))
 
+(define-method (code:type->string (o <extern>))
+  (string-join (ast:full-name o) "_"))
+
 (define-method (code:reply-var (o <type>))
   (let ((type (code:type->string o)))
     (simple-format #f "dzn_reply_~a" type)))
@@ -394,59 +392,6 @@
   (let ((compound (make <compound> #:elements (list o))))
     (clone compound #:parent (.parent o))))
 
-
-;;;
-;;; Calling context.
-;;;
-(define-method (code:add-calling-context-formal (o <formals>))
-  (if (not (%calling-context)) o
-      (let* ((cc-formal (make <formal>
-                          #:name "dzn_cc"
-                          #:direction 'inout
-                          #:type.name (make <scope.name>
-                                        #:ids '("*calling-context*"))))
-             (cc-formal (clone cc-formal #:parent o))
-             (lst (cons cc-formal (ast:formal* o))))
-        (clone o #:elements lst))))
-
-(define-method (code:formal* (o <formals>))
-  (ast:formal* (code:add-calling-context-formal o)))
-
-(define-method (code:formal* (o <signature>))
-  (code:formal* (.formals o)))
-
-(define-method (code:formal* (o <function>))
-  (code:formal* (.signature o)))
-
-(define-method (code:formal* (o <event>))
-  (code:formal* (.signature o)))
-
-(define-method (code:formal* (o <action>))
-  (code:formal* (.event o)))
-
-(define-method (code:formal* (o <trigger>))
-  (code:formal* (.formals o)))
-
-(define-method (code:formal* (o <on>))
-  (code:formal*
-   (let* ((trigger ((compose car ast:trigger*) o))
-          (formals ((compose .formals .signature) trigger))
-          (event (.event trigger))
-          (elements (map (cute clone <> #:name <>)
-                         (ast:formal* event)
-                         (map .name (ast:formal* formals)))))
-     (clone formals #:elements elements))))
-
-(define (code:add-calling-context-argument arguments)
-  (if (%calling-context) (cons "dzn_cc" arguments)
-      arguments))
-
-(define-method (code:argument* (o <call>))
-  (code:add-calling-context-argument (ast:argument* o)))
-
-(define-method (code:argument* (o <action>))
-  (code:add-calling-context-argument (ast:argument* o)))
-
 (define-method (code:number-formals formals)
   (map (cute clone <> #:name <>)
        formals (iota (length formals))))
@@ -531,8 +476,9 @@
     (and=> (regexp-exec flush-regexp o)
            (cute match:substring <> 0)))
   (define (trigger? o)
-    (and=> (regexp-exec trigger-regexp o)
-           (cute match:substring <> 1)))
+    (and (string? o)
+         (and=> (regexp-exec trigger-regexp o)
+                (cute match:substring <> 1))))
   (define (action? o)
     (and=> (regexp-exec action-regexp o)
            (cute match:substring <> 1)))
@@ -568,8 +514,7 @@
         (and=> (regexp-exec enum-regexp event)
                (compose makreel->enum
                         (cute match:substring <> 1)))
-        ;; HACK for debugging
-        (pke "EVENT NOT FOUND" event)))
+        (throw 'programming-error (format #f "event not found ~s\n" event))))
   (define (edge->transition edge)
     (let* ((label ((disjoin trigger? action? reply? flush?) (edge-label edge)))
            (label (event->prefix label)))
@@ -754,33 +699,65 @@
 ;;;
 ;;; Transform.
 ;;;
-(define-method (code:add-calling-context (o <root>))
+(define (code:add-calling-context o)
+  "Add extra parameter of type -c, --calling-context=TYPE for every event,
+and pass it as argument accordingly."
   (if (not (%calling-context)) o
-      (let* ((name (make <scope.name> #:ids '("*calling-context*")))
-             (extern (make <extern>
-                       #:name name
-                       #:value (%calling-context))))
-        (clone o #:elements (cons extern (ast:top* o))))))
+      (match o
+        (($ <arguments>)
+         (let* ((arguments (ast:argument* o))
+                (arguments (cons "dzn_cc" arguments)))
+           (clone o #:elements arguments)))
+        (($ <formals>)
+         (let* ((type-name (make <scope.name> #:ids '("*calling-context*")))
+                (cc-formal (make <formal>
+                             #:name "dzn_cc"
+                             #:direction 'inout
+                             #:type.name type-name))
+                (formals (cons cc-formal (ast:formal* o))))
+           (clone o #:elements formals)))
+        (($ <root>)
+         (let* ((name (make <scope.name> #:ids '("*calling-context*")))
+                (data (make <data> #:value (%calling-context)))
+                (extern (make <extern> #:name name #:value data))
+                (o (tree-map code:add-calling-context o)))
+           (clone o #:elements (cons extern (ast:top* o)))))
+        (($ <system>)
+         o)
+        ((? (%normalize:short-circuit?))
+         o)
+        ((? (is? <ast>))
+         (tree-map code:add-calling-context o))
+        (_
+         o))))
 
 (define (code:annotate-shells o)
-  (let ((shell? (and (is-a? o <system>)
-                     (member (ast:dotted-name o) (%shell)))))
-    (if (not shell?) o
-        (let ((shell (make <shell-system>
-                       #:ports (.ports o)
-                       #:name (.name o)
-                       #:instances (.instances o)
-                       #:bindings (.bindings o))))
-          (clone shell #:parent (.parent o))))))
+  (match o
+    ((and ($ <system>)
+          (? (compose (cute member <> (%shell))
+                      ast:dotted-name)))
+     (let ((shell (make <shell-system>
+                    #:ports (.ports o)
+                    #:name (.name o)
+                    #:instances (.instances o)
+                    #:bindings (.bindings o))))
+       (clone shell #:parent (.parent o))))
+    ((? (%normalize:short-circuit?))
+     o)
+    ((? (is? <namespace>))
+     (tree-map code:annotate-shells o))
+    (_ o)))
 
 (define (code:normalize- ast)
   (parameterize ((%normalize:short-circuit? code:short-circuit?))
     ((compose
+      code:annotate-shells
       add-reply-port
+      (binding-into-blocking)
       normalize:event+illegals
       remove-otherwise
-      (binding-into-blocking)
-      code:add-calling-context)
+      code:add-calling-context
+      (add-explicit-temporaries #:call-only? #t))
      ast)))
 
 (define (code:normalize ast)
@@ -788,20 +765,10 @@
 normalizations."
   (let ((root (code:normalize- ast)))
     (when (> (dzn:debugity) 1)
+      (debug "normalized root:")
       (ast:pretty-print root (current-error-port)))
     root))
 
-(define (code:normalize+determinism ast)
-  "Code:normalize, plus add-determinism-temporaries."
-  (parameterize ((%normalize:short-circuit? code:short-circuit?))
-    (let ((root ((compose
-                  add-determinism-temporaries
-                  code:normalize-)
-                 ast)))
-      (when (> (dzn:debugity) 1)
-        (ast:pretty-print root (current-error-port)))
-      root)))
-
 (define (code:short-circuit? o)
   (match o
     ((or ($ <foreign>) ($ <system>))
@@ -815,8 +782,15 @@ normalizations."
 ;;;
 ;;; Entry point.
 ;;;
-(define* (code ast #:key (ast-> 'ast->) calling-context dir language locations?
-               model shell)
+(define* (code ast #:key (ast-> 'ast->)
+               calling-context
+               dir
+               empty-files?
+               language
+               locations?
+               model
+               shell
+               verbose?)
   (let* ((module (resolve-module `(dzn code language ,(string->symbol language))))
          (ast-> (false-if-exception (module-ref module ast->))))
     (unless ast->
@@ -825,4 +799,5 @@ normalizations."
     (parameterize ((%calling-context calling-context)
                    (%locations? locations?)
                    (%shell shell))
-      (ast-> ast #:dir dir #:model model))))
+      (ast-> ast #:dir dir #:empty-files? empty-files? #:model model
+             #:verbose? verbose?))))
