@@ -114,15 +114,6 @@
   "Return a full name of MODEL, separated with dots, with ticks removed."
   (string-join (map untick (ast:full-name o)) "."))
 
-(define-method (makreel:get-model (o <root>))
-  (define (named? o)
-    (equal? (makreel:unticked-dotted-name o) (%model-name)))
-  (let ((model (and (%model-name)
-                    (find named? (ast:model** o)))))
-    (or model
-        (find (is? <component>) (ast:model** o))
-        (filter (is? <interface>) (ast:model** o)))))
-
 
 ;;;
 ;;; Accessors.
@@ -520,7 +511,7 @@
                               #:variable.name name)))
          (cons* statement action-reply (add-action-reply rest))))
       ((statement rest ...)
-       (cons statement (add-action-reply rest)))
+       (cons (add-action-reply statement) (add-action-reply rest)))
       (_
        o)))
 
@@ -531,9 +522,8 @@
      (clone o #:statement (makreel:add-action-reply (.statement o))))
     (($ <guard>)
      (clone o #:statement (makreel:add-action-reply (.statement o))))
-    ((and (? ast:imperative?) ($ <compound>))
-     (let ((elements (add-action-reply (ast:statement* o))))
-       (clone o #:elements elements)))
+    (($ <compound>)
+     (add-action-reply o))
     (($ <behavior>)
      (clone o
             #:statement (makreel:add-action-reply (.statement o))
