@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2022, 2023, 2024 Janneke Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2023, 2024, 2025 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2021, 2022, 2023, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2023, 2024, 2025, 2026 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -28,10 +28,11 @@
   #:use-module (scmackerel code)
   #:use-module (scmackerel header)
 
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast)
   #:use-module (dzn code)
-  #:use-module (dzn code goops)
+  #:use-module (dzn code ast)
+  #:use-module (dzn code shared)
   #:use-module (dzn code scmackerel code)
   #:use-module (dzn code language dzn)
   #:use-module (dzn code language c++)
@@ -258,10 +259,10 @@ std::basic_ostream<Char, Traits> &")
 
 (define-method (ast->code (o <action>))
   (let* ((event (.event o))
-         (formals (code:formal* (.event o)))
+         (formals (ast:formal* (.event o)))
          (event-name (.event.name o))
          (action-name (code:event-name o))
-         (arguments (code:argument* o))
+         (arguments (ast:argument* o))
          (arguments (map c++:ast->expression arguments)))
     (sm:call (name (string-append (%member-prefix) action-name))
              (arguments arguments))))
@@ -320,6 +321,9 @@ std::basic_ostream<Char, Traits> &")
       (ast->code (code:wrap-compound o)))
      (else
       (let* ((type (ast:type o))
+             (type (if (not (is-a? (.expression o) <data>)) type
+                       ((compose .type .signature .event car ast:trigger*)
+                        (ast:parent o <on>))))
              (reply-port (.port o))
              (port-name (.name reply-port))
              (out-binding (string-append "(*" (%member-prefix)
@@ -430,8 +434,11 @@ std::basic_ostream<Char, Traits> &")
 ;;;
 (define-method (interface->sm:statements-unmemoized (o <interface>))
   (define (event->slot event)
-    (let ((type (code:type-name (ast:type event)))
-          (formals (code:formal* event)))
+    (let* ((type (ast:type event))
+           (extern? (is-a? type <extern>))
+           (type (if (not extern?) (code:type-name type)
+                     (format #f "dzn::data<~a>" (code:type-name type))))
+           (formals (ast:formal* event)))
       (sm:variable
        (type (simple-format #f "dzn::~a::event<~a>"
                             (code:direction event)
@@ -445,7 +452,7 @@ std::basic_ostream<Char, Traits> &")
               (sm:statement*
                (format #f "throw dzn::binding_error (this->dzn_meta, ~s)"
                        event-name)))))
-  (let* ((shared initial (code:shared-state o))
+  (let* ((shared initial (if (%no-constraint?) (values #f 0) (code:shared-state o)))
          (public-enums (code:public-enum* o))
          (enums (append public-enums (code:enum* o)))
          (interface
@@ -477,15 +484,6 @@ std::basic_ostream<Char, Traits> &")
       (let* ((expression (.expression ast))
              (value (code:shared-value expression)))
         (sm:assign* (.variable.name ast) value)))
-    (define (state->if shared else)
-      (let* ((expression (sm:equal* "dzn_state" (.state shared)))
-             (assignments (ast:statement* (.assign shared)))
-             (assignments (map ast->assign assignments)))
-        (sm:if* expression
-                (match assignments
-                  ((assign) assign)
-                  ((assigns ...) (sm:compound* assigns)))
-                else)))
 
     (define (value->init value)
       (simple-format #f "~s" value))
@@ -786,9 +784,13 @@ std::basic_ostream<Char, Traits> &")
      (name (.name port))
      (expression (sm:member* (simple-format #f "dzn_locator.get< ~a> ()" type)))))
   (define (type->reply-variable o)
-    (sm:variable
-     (type (string-append (code:type-name o) "*"))
-     (name (code:reply-var o))))
+    (let ((extern? (is-a? o <extern>)))
+      (sm:variable
+       (type (string-append
+              (if (not extern?) (code:type-name o)
+                  (format #f "dzn::data<~a>" (code:type-name o)))
+              "*"))
+       (name (code:reply-var o)))))
   (define (port->out-binding port)
     (sm:variable
      (type "std::function<void ()>*")
@@ -813,11 +815,12 @@ std::basic_ostream<Char, Traits> &")
            (direction (code:direction event))
            (event-name (.name event))
            (event-name-string (simple-format #f "~s" event-name))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (arguments (map .name formals))
            (in-formals (filter ast:in? formals))
            (in-arguments (map .name in-formals))
            (formals (map c++:->formal formals))
+           (out-binding (code:out-binding port))
            (return-type (ast:type trigger))
            (type (code:type-name return-type))
            (typed? (ast:typed? trigger))
@@ -863,7 +866,7 @@ std::basic_ostream<Char, Traits> &")
                       (trigger (car (ast:trigger* on)))
                       (statement (.statement on)))
                  (values trigger (ast->code statement))))))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (return-type (ast:type trigger)))
       (sm:method (struct component)
                  (type (if (not (is-a? o <foreign>)) "void"
@@ -875,7 +878,7 @@ std::basic_ostream<Char, Traits> &")
   (define (function->method component function)
     (let* ((type (code:type-name (ast:type function)))
            (name (.name function))
-           (formals (code:formal* function))
+           (formals (ast:formal* function))
            (statement (.statement function))
            (statement (ast->code statement)))
       (sm:method (struct component)
@@ -899,7 +902,7 @@ std::basic_ostream<Char, Traits> &")
                       ports))))
   (define (out->method component trigger)
     (let* ((arguments (map c++:ast->expression
-                           (code:formal* trigger)))
+                           (ast:formal* trigger)))
            (lambda (sm:function
                     (captures `("this" ,@arguments))
                     (statement
@@ -912,7 +915,7 @@ std::basic_ostream<Char, Traits> &")
       (sm:method (struct component)
                  (type "void")
                  (name (code:event-slot-name trigger))
-                 (formals (map c++:->formal (code:formal* trigger)))
+                 (formals (map c++:->formal (ast:formal* trigger)))
                  (statement
                   (sm:compound*
                    (sm:call
@@ -1237,7 +1240,7 @@ std::basic_ostream<Char, Traits> &")
              (port-name (.name port))
              (event (.event trigger))
              (event-name (.name event))
-             (formals (code:formal* trigger))
+             (formals (ast:formal* trigger))
              (arguments (map .name formals))
              (in-formals (filter ast:in? formals))
              (in-arguments (map .name in-formals))
@@ -1276,7 +1279,7 @@ std::basic_ostream<Char, Traits> &")
              (event (.event trigger))
              (event-name (.name event))
              (direction (symbol->string (.direction event)))
-             (formals- (code:formal* trigger))
+             (formals- (ast:formal* trigger))
              (arguments (map .name formals-)))
         (sm:assign*
          (string-append ;;; XXX FIXME code:foo-name
@@ -1399,7 +1402,7 @@ std::basic_ostream<Char, Traits> &")
     (let* ((port (.port.name trigger))
            (event (.event.name trigger))
            (direction (ast:direction trigger))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (formals (map (cute clone <> #:name #f) formals))
            (formals (map c++:->formal formals))
            (system-port (string-append "c.system." port))
@@ -1444,7 +1447,7 @@ std::basic_ostream<Char, Traits> &")
            (port-event (simple-format #f "~a.~a" port event))
            (flush-event (simple-format #f "~a.<flush>" port))
            (flush-string (simple-format #f "~s" flush-event))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (formals (map (cute clone <> #:name #f) formals))
            (formals (map c++:->formal formals))
            (port (.port trigger)))
@@ -1511,7 +1514,7 @@ std::basic_ostream<Char, Traits> &")
   (define (void-provides-in->init trigger)
     (let* ((port (.port.name trigger))
            (event (.event.name trigger))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (formals (code:number-formals formals))
            (arguments (map code:number-argument formals))
            (out-formals (filter (negate ast:in?) formals))
@@ -1536,7 +1539,7 @@ std::basic_ostream<Char, Traits> &")
   (define (typed-provides-in->init trigger)
     (let* ((port (.port.name trigger))
            (event (.event.name trigger))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (formals (code:number-formals formals))
            (arguments (map code:number-argument formals))
            (out-formals (filter (negate ast:in?) formals))
@@ -1602,7 +1605,7 @@ std::basic_ostream<Char, Traits> &")
            (event (.event.name trigger))
            (port-event (simple-format #f "~a.~a" port event))
            (port-event-string (simple-format #f "~s" port-event))
-           (formals (code:formal* trigger)))
+           (formals (ast:formal* trigger)))
       (sm:generalized-initializer-list*
        port-event-string
        (sm:function
