@@ -1,7 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2023, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2025 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2025 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -28,10 +29,10 @@
   #:use-module (scmackerel c)
   #:use-module (scmackerel c-header)
 
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast)
   #:use-module (dzn code)
-  #:use-module (dzn code goops)
+  #:use-module (dzn code ast)
   #:use-module (dzn code scmackerel code)
   #:use-module (dzn code language dzn)
   #:use-module (dzn code language c)
@@ -107,8 +108,7 @@
   (let* ((type (ast:type o))
          (interface (.type (.port o)))
          (formals (ast:formal* o))
-         (types (map .type formals))
-         (types (map c:type-name types)))
+         (types (map c:formal-type-name formals)))
     (simple-format #f "(~a (*) (~a*~a))"
                    (c:type-name type)
                    (c:type-name interface)
@@ -291,7 +291,7 @@
 
 (define-method (ast->code (o <action>))
   (let* ((action-name (c:event-name o))
-         (arguments (code:argument* o))
+         (arguments (ast:argument* o))
          (arguments (map ast->expression arguments))
          (port-name (.port.name o))
          (typed? (ast:typed? o)))
@@ -304,7 +304,7 @@
   (let* ((model (ast:parent o <model>))
          (model-name (c:type-name model))
          (name (string-append model-name "_" (.function.name o)))
-         (arguments (map ast->expression (code:argument* o)))
+         (arguments (map ast->expression (ast:argument* o)))
          (arguments (cons "self" arguments)))
     (sm:call
      (name name)
@@ -365,6 +365,9 @@
       (ast->code (code:wrap-compound o)))
      (else
       (let* ((type (ast:type o))
+             (type (if (not (is-a? (.expression o) <data>)) type
+                       ((compose .type .signature .event car ast:trigger*)
+                        (ast:parent o <on>))))
              (reply-port (.port o))
              (port-name (.name reply-port))
              (out-binding (string-append (%member-prefix)
@@ -450,7 +453,7 @@
 (define-method (interface->sm:statements-unmemoized (o <interface>))
   (define (event->slot event)
     (let ((type (c:type-name (ast:type event)))
-          (formals (code:formal* event)))
+          (formals (ast:formal* event)))
       (sm:function
        (type type)
        (name (string-append "(*" (.name event) ")"))
@@ -588,7 +591,7 @@
                       (trigger (car (ast:trigger* on)))
                       (statement (.statement on)))
                  (values trigger (ast->code statement))))))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (arguments (map .name formals))
            (formals (map c:->formal formals))
            (typed? (ast:typed? trigger))
@@ -648,7 +651,7 @@
                 (name "dzn_runtime_trace_out")
                 (arguments
                  (list "&port->meta"
-                       (if (not typed?)
+                       (if (or (not typed?) (is-a? (ast:type trigger) <extern>))
                            (simple-format #f "~s" "return")
                            (sm:call
                             (name (string-append "dzn_" reply-type
@@ -674,7 +677,7 @@
                       (trigger (car (ast:trigger* on)))
                       (statement (.statement on)))
                  (values trigger (ast->code statement))))))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (port (.port trigger))
            (interface (c:type-name (.type port)))
            (event-name (.event.name trigger))
@@ -718,7 +721,7 @@
                             "_"
                             (code:event-slot-name trigger)))))
             ,(sm:assign* "dzn_c.self" "self")
-            ,@(let ((formals (code:formal* trigger)))
+            ,@(let ((formals (ast:formal* trigger)))
                 (map formal->assign formals (iota (length formals))))
             ,(sm:call (name "dzn_runtime_enqueue")
                       (arguments
@@ -734,7 +737,7 @@
   (define (function->method component function)
     (let* ((type (code:type-name (ast:type function)))
            (name (.name function))
-           (formals (code:formal* function))
+           (formals (ast:formal* function))
            (statement (.statement function))
            (statement (ast->code statement)))
       (sm:method (struct component)
@@ -1173,7 +1176,7 @@
            (event (.event.name trigger))
            (port-event (simple-format #f "~a.~a" port event))
            (port-event-string (simple-format #f "~s" port-event))
-           (formals (code:formal* trigger)))
+           (formals (ast:formal* trigger)))
       (sm:assign* (string-append "m->" (c:event-name trigger))
                   (string-append
                    (c:log-event-type-cast trigger)
