@@ -1,6 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2022, 2023, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2025 Rutger (regtur) van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2025 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -25,10 +27,11 @@
   #:use-module (ice-9 match)
   #:use-module (scmackerel cs)
 
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast)
   #:use-module (dzn code)
-  #:use-module (dzn code goops)
+  #:use-module (dzn code ast)
+  #:use-module (dzn code shared)
   #:use-module (dzn code scmackerel code)
   #:use-module (dzn code language dzn)
   #:use-module (dzn code language cs)
@@ -145,16 +148,16 @@
 
 (define-method (ast->code (o <action>))
   (let* ((action-name (code:event-name o))
-         (formals (code:formal* (.event o)))
-         (arguments (code:argument* o))
+         (formals (ast:formal* (.event o)))
+         (arguments (ast:argument* o))
          (arguments (map ast->expression arguments formals))
          (action (sm:call (name (string-append (%member-prefix) action-name))
                           (arguments arguments))))
     action))
 
 (define-method (ast->code (o <call>))
-  (let* ((formals (code:formal* (.function o)))
-         (arguments (code:argument* o))
+  (let* ((formals (ast:formal* (.function o)))
+         (arguments (ast:argument* o))
          (arguments (map ast->expression arguments formals)))
     (sm:call-method
      (name (.function.name o))
@@ -167,6 +170,9 @@
       (ast->code (code:wrap-compound o)))
      (else
       (let* ((type (ast:type o))
+             (type (if (not (is-a? (.expression o) <data>)) type
+                       ((compose .type .signature .event car ast:trigger*)
+                        (ast:parent o <on>))))
              (reply-port (.port o))
              (port-name (.name reply-port))
              (out-binding (string-append (%member-prefix)
@@ -279,7 +285,7 @@
     (let* ((name (.name event))
            (signature-name (string-append "signature_" name))
            (type (code:type-name (ast:type event)))
-           (formals (code:formal* event)))
+           (formals (ast:formal* event)))
       (sm:method
        (struct port)
        (type (string-append "delegate " type))
@@ -418,7 +424,7 @@
            (event (.event trigger))
            (event-name (.name event))
            (event-name-string (simple-format #f "~s" event-name))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (arguments (map cs:->argument formals))
            (out-formals (filter (negate ast:in?) formals))
            ;; (out-arguments (map .name out-formals))
@@ -497,7 +503,7 @@
                       (trigger (car (ast:trigger* on)))
                       (statement (.statement on)))
                  (values trigger (ast->code statement))))))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (formals (map cs:->formal formals))
            (type (ast:type trigger)))
       (sm:method (struct component)
@@ -509,7 +515,7 @@
   (define (function->method component function)
     (let* ((type (code:type-name (ast:type function)))
            (name (.name function))
-           (formals (code:formal* function))
+           (formals (ast:formal* function))
            (formals (map cs:->formal formals))
            (statement (.statement function))
            (statement (ast->code statement)))
@@ -829,7 +835,7 @@
              (port-name (.name port))
              (event (.event trigger))
              (event-name (.name event))
-             (formals (code:formal* trigger))
+             (formals (ast:formal* trigger))
              (arguments (map cs:->argument formals))
              (out-formals (filter (negate ast:in?) formals))
              ;; (out-arguments (map .name out-formals))
@@ -882,7 +888,7 @@
              (other-end (ast:other-end-point port))
              (event (.event trigger))
              (event-name (.name event))
-             (formals (code:formal* trigger))
+             (formals (ast:formal* trigger))
              (arguments (map cs:->argument formals))
              (out-formals (filter (negate ast:in?) formals))
              (formals (map cs:->formal formals))
@@ -1034,7 +1040,7 @@
     (let* ((port (.port.name trigger))
            (event (.event.name trigger))
            (direction (ast:direction trigger))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (out-formals (filter (negate ast:in?) formals))
            (formals (map cs:->formal formals))
            (system-port (string-append "c.system." port))
@@ -1078,7 +1084,7 @@
            (port-event (simple-format #f "~a.~a" port event))
            (flush-event (simple-format #f "~a.<flush>" port))
            (flush-string (simple-format #f "~s" flush-event))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (formals (map cs:->formal formals))
            (call-out
             (sm:call
@@ -1154,7 +1160,7 @@
   (define (void-provides-in->init trigger)
     (let* ((port (.port.name trigger))
            (event (.event.name trigger))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (formals (code:number-formals formals))
            (arguments (map cs:number-argument formals))
            (out-formals (filter (negate ast:in?) formals))
@@ -1193,7 +1199,7 @@
            (port-sm:return-string (simple-format #f "~s" port-return))
            (port-prefix (simple-format #f "~a." port))
            (event-slot (string-append "c.system." (code:event-name trigger)))
-           (formals (code:formal* trigger))
+           (formals (ast:formal* trigger))
            (invoke (sm:code->string (sm:call (name event-slot)
                                              (arguments (iota (length formals)))))))
       (sm:call
@@ -1255,7 +1261,7 @@
            (event (.event.name trigger))
            (port-event (simple-format #f "~a.~a" port event))
            (port-event-string (simple-format #f "~s" port-event))
-           (formals (code:formal* trigger)))
+           (formals (ast:formal* trigger)))
       (sm:call
        (name "lookup.Add")
        (arguments
