@@ -1,9 +1,9 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021, 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2019, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
-;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026 Rutger (regtur) van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2021, 2024, 2025 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -63,14 +63,12 @@
 (define (peg:imports string)
   ;; . 'import'? eagerly SKIP anything with import in it
   (define-peg-string-patterns
-    "root <- (dollars / import / SKIP)*
-dollars < DOLLAR (!DOLLAR .)* DOLLAR#
+    "root <- import*
 file-name <- (!SEMICOLON !DOLLAR .)+
 import <-- IMPORT file-name SEMICOLON
 DOLLAR < '$'
 IMPORT < 'import' ![a-zA-Z_0-9]
-SEMICOLON < ';'
-SKIP < . 'import'?")
+SEMICOLON < ';'")
   (peg:tree (match-pattern root string)))
 
 (define peg:skip-parse peg-skip)
@@ -165,9 +163,14 @@ SKIP < . 'import'?")
   (define-peg-pattern dollars all dollars-no-skip)
 
   (define-peg-string-patterns
-    "root <-- (import / dollars / type
-               / namespace / interface / component
-               / EOF)#*
+    "root <-- (EOF
+            / imports EOF
+            / imports definitions
+            / definitions)#
+
+imports <-- import+
+
+definitions <-- (dollars / type / namespace / interface / component / EOF)#+
 
 import <-- IMPORT file-name SEMICOLON#
   file-name <- (!SEMICOLON .)+
@@ -188,17 +191,17 @@ type <- enum / int / extern
 namespace <-- NAMESPACE compound-name# BRACE-OPEN# namespace-root BRACE-CLOSE#
   namespace-root <-- (type / namespace / interface / component / &BRACE-CLOSE)#*
 
-interface <-- INTERFACE reset-event-names reset-port-names scoped-name#
+interface <-- INTERFACE scoped-name#
               BRACE-OPEN# types-and-events# (behavior / &BRACE-CLOSE)#
-              BRACE-CLOSE#
+              BRACE-CLOSE# reset-event-names
 
   types-and-events <-- (type / event / &behavior / &BRACE-CLOSE)#*
     event <-- direction type-name# event-name#
               enter-frame formals# exit-frame SEMICOLON#
       direction <-- IN / OUT
 
-component <-- COMPONENT reset-port-names reset-event-names scoped-name#
-              BRACE-OPEN# ports# body# BRACE-CLOSE#
+component <-- COMPONENT scoped-name#
+              BRACE-OPEN# ports# body# BRACE-CLOSE# reset-port-names
   body <- behavior / system / &BRACE-CLOSE
     system <-- SYSTEM BRACE-OPEN# instances-and-bindings BRACE-CLOSE#
       instances-and-bindings <-- (binding / instance)*
@@ -216,19 +219,25 @@ component <-- COMPONENT reset-port-names reset-event-names scoped-name#
 
 type-name <-- compound-name / BOOL / VOID
 
-behavior <-- BEHAVIOR (name)? behavior-compound
+behavior <-- BEHAVIOR behavior-compound
 
   behavior-compound
     <-- BRACE-OPEN# enter-frame behavior-statements BRACE-CLOSE# exit-frame
 
     behavior-statements
-       <-- (function / variable / declarative-statement / type / &BRACE-CLOSE)#*
+       <-- (expression-function / function / variable / declarative-statement /
+            type / &BRACE-CLOSE)#*
 
       function
-         <-- type-name name &(formals BRACE-OPEN)
+         <-- type-name (!is-event name / duplicate-identifier) &(formals BRACE-OPEN)
              enter-frame formals compound# exit-frame
 
-declarative-statement <- on / blocking / guard / compound
+      expression-function
+         <-- type-name (!is-event name / duplicate-identifier)
+             &PAREN-OPEN formals ASSIGN expression# SEMICOLON#
+
+declarative-statement <- on / blocking / guard / compound / invariant
+
   on <-- ON (illegal-triggers COLON illegal
              / enter-frame triggers# COLON# (statement / !unknown-identifier)#
              exit-frame)
@@ -253,16 +262,19 @@ compound
   statement
      <- (declarative-statement / imperative-statement / !unknown-identifier)#
 
+invariant
+   <-- INVARIANT expression# SEMICOLON#
+
 imperative-statement
   <- variable / assign / if-statement / illegal / return / skip-statement
-     / compound / reply / defer / action-or-call / interface-action SEMICOLON#
+     / compound / reply / defer / action-or-call
 
-  defer <-- DEFER arguments? imperative-statement
-
-  interface-action <-- is-event
+  defer <-- DEFER defer-arguments? imperative-statement
+  defer-arguments <-- PAREN-OPEN (defer-argument (&PAREN-CLOSE / COMMA#))* PAREN-CLOSE#
+  defer-argument <-- var / shared-var / unknown-identifier
 
   action-or-call <- (action / call) SEMICOLON#
-    action <-- is-port DOT name# arguments
+    action <-- is-port DOT name# arguments / is-event !arguments
     call <-- name arguments
       arguments <-- PAREN-OPEN (argument (&PAREN-CLOSE / COMMA#))* PAREN-CLOSE#
         argument <-- expression
@@ -288,7 +300,10 @@ imperative-statement
     <-- type-name add-var (ASSIGN expression#)? SEMICOLON#
 
 expression
-  <-- or-expression
+  <-- implies-expression
+
+implies-expression
+  <- or-expression IMPLIES implies-expression# / or-expression
 
 or-expression
   <- and-expression OR or-expression# / and-expression
@@ -304,11 +319,11 @@ plus-min-expression
   <- not-expression (PLUS / MINUS) plus-min-expression# / not-expression
 
 not-expression
-  <- minus / not / group / dollars / literal / !(is-port / var) enum-literal
+  <- minus / negation / group / dollars / literal / !(is-port / var) enum-literal
      / field-test / action / shared-field-test / shared-var / call / var
-     / interface-action / unknown-identifier
+     / unknown-identifier
 
-not <-- NOT not-expression#
+negation <-- NOT not-expression#
 minus <-- UMINUS not-expression#
 enum-literal <-- global? scope name
 field-test <-- var DOT name#
@@ -330,6 +345,7 @@ global <-- DOT
 identifier <- !KEYWORD [a-zA-Z_] [a-zA-Z_0-9]*
 
 unknown-identifier <-- identifier
+duplicate-identifier <-- identifier
 
 otherwise <-- OTHERWISE
 provides <-- PROVIDES
@@ -364,6 +380,7 @@ LESS                <-  '<'
 LESS-EQUAL          <-  '<='
 GREATER             <-  '>'
 GREATER-EQUAL       <-  '>='
+IMPLIES             <-  '=>'
 PLUS                <-  '+'
 MINUS               <-  '-'
 NOT                 <   '!'
@@ -388,6 +405,7 @@ INEVITABLE          <- 'inevitable' ![a-zA-Z_0-9]
 INJECTED            <  'injected' ![a-zA-Z_0-9]
 INOUT               <- 'inout' ![a-zA-Z_0-9]
 INTERFACE           <  'interface' ![a-zA-Z_0-9]
+INVARIANT           <  'invariant' ![a-zA-Z_0-9]
 NAMESPACE           <  'namespace' ![a-zA-Z_0-9]
 ON                  <  'on' ![a-zA-Z_0-9]
 OPTIONAL            <- 'optional' ![a-zA-Z_0-9]
@@ -422,6 +440,7 @@ KEYWORD <
   / 'injected' ![a-zA-Z_0-9]
   / 'inout' ![a-zA-Z_0-9]
   / 'interface' ![a-zA-Z_0-9]
+  / 'invariant' ![a-zA-Z_0-9]
   / 'namespace' ![a-zA-Z_0-9]
   / 'on' ![a-zA-Z_0-9]
   / 'optional' ![a-zA-Z_0-9]
