@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2018, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2018, 2020, 2021, 2022, 2023, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2021 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
@@ -23,17 +23,18 @@
 ;;; Code:
 
 (define-module (dzn trace)
-  #:use-module (ice-9 match)
-  #:use-module (ice-9 pretty-print)
-  #:use-module (ice-9 regex)
-
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-9 gnu)
   #:use-module (srfi srfi-26)
 
-  #:use-module (dzn peg)
+  #:use-module (ice-9 match)
+  #:use-module (ice-9 pretty-print)
+  #:use-module (ice-9 regex)
+
   #:use-module (dzn parse)
-  #:use-module (dzn parse util)
+  #:use-module (dzn parse location)
+  #:use-module (dzn peg)
+  #:use-module (dzn peg util)
   #:use-module (dzn misc)
 
   #:use-module (json)
@@ -911,7 +912,7 @@ ws               <   [ \t]
   (let* ((header (find header? steps))
          (r-steps (reverse steps))
          (eligible (find eligible? r-steps))
-         (eligible (and eligible (eligible-sexp eligible)))
+         (eligible (if eligible (eligible-sexp eligible) '()))
          (labels (find labels? r-steps))
          (labels (if labels (labels-sexp labels) '()))
          (header (header-sexp header))
@@ -963,8 +964,7 @@ ws               <   [ \t]
         (let ((lifelines (map (cute instance->lifeline <> activities labels eligible) instances))
               (trail (steps->trail steps)))
           (scm->json-string
-           `((working-directory . ,(getcwd))
-             (trail     . ,(list->vector trail))
+           `((trail     . ,(list->vector trail))
              (lifelines . ,(list->vector (map lifeline->scm lifelines)))
              (events    . ,(list->vector (map lifeline-event->scm events)))
              (states    . ,(list->vector (map (cute lifeline-state->json-scm sut-name <>) states)))))))
