@@ -1,6 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2022, 2023 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2022, 2023, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2022, 2024 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -30,34 +31,51 @@
 
   #:use-module (ice-9 match)
 
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast normalize)
   #:use-module (dzn ast)
   #:use-module (dzn code)
   #:use-module (dzn code language makreel)
+  #:use-module (dzn command-line)
   #:use-module (dzn misc)
   #:use-module (dzn vm normalize)
 
-  #:export (inline-functions
+  #:export (add-calling-context
+            add-explicit-temporaries
+            inline-functions
             normalize:compounds-wrap)
   #:re-export (add-function-return
-               add-determinism-temporaries
-               add-explicit-temporaries
                code:normalize
-               code:normalize+determinism
                extract-call
+               inline-expression-functions
+               invariant:include-guard-expressions
                makreel:normalize
                normalize:compounds
                normalize:event
                normalize:state
                purge-data
-               remove-otherwise
                remove-behavior
+               remove-invariant
+               remove-otherwise
                simplify-guard-expressions
-               split-complex-expressions
                tag-imperative-blocks
                vm:normalize))
 
+(define (add-calling-context o)
+  "Add extra parameter of type -c, --calling-context=TYPE for every event,
+and pass it as argument accordingly."
+  (let ((calling-context (false-if-exception
+                          (command-line:get 'calling-context))))
+    (unless calling-context
+      (format (current-error-port) "warning: no -c,--calling-context set.\n"))
+    (parameterize ((%calling-context calling-context))
+      (code:add-calling-context o))))
+
+(define (add-explicit-temporaries o)
+  "Make implicit temporary values in action, call, if, reply, and return
+expressions explicit."
+  (((@ (dzn ast normalize) add-explicit-temporaries)) o))
+
 (define (normalize:compounds-wrap o)
   "Like normalize:compounds and wrap singleton top level imperative
 statements in a compound."
