@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2018, 2019, 2020, 2021, 2022, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018, 2019, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
@@ -22,41 +22,17 @@
 (define-module (dzn vm runtime)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
+
   #:use-module (ice-9 match)
-  #:use-module (dzn misc)
   #:use-module (ice-9 poe)
-  #:use-module ((oop goops) #:renamer (lambda (x) (if (member x '(<port> <foreign>)) (symbol-append 'goops: x) x)))
+
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast)
-  #:use-module (dzn goops)
+  #:use-module (dzn misc)
+
   #:export (%sut
             %instances
-            ast->runtime:instance
-            runtime:%sut-model
-            runtime:ast-model
-            runtime:get-sut
-            runtime:boundary-port?
-            runtime:component-instance?
-            runtime:container-eq?
-            runtime:container-path
-            runtime:dotted-name
-            runtime:foreign-instance?
-            runtime:instance->path
-            runtime:instance->string
-            runtime:kind
-            runtime:other-instance+port
-            runtime:other-port
-            runtime:path->instance
-            runtime:port
-            runtime:port*
-            runtime:port-instance?
-            runtime:port-name->instance
-            runtime:provides-instance?
-            runtime:runtime-port*
-            runtime:runtime-provides-port*
-            runtime:runtime-requires-port*
-            runtime:requires-instance?
-            runtime:system-instance?
-            runtime:create-instances
+
             <runtime:component>
             <runtime:component-model>
             <runtime:foreign>
@@ -65,9 +41,27 @@
             <runtime:system>
             <runtime:trigger>
             <runtime>
+
             .container
             .boundary?
-            .rank))
+
+            ast->runtime:instance
+            runtime:%sut-model
+            runtime:ast-model
+            runtime:get-sut
+            runtime:boundary-port?
+            runtime:create-instances
+            runtime:dotted-name
+            runtime:instance->path
+            runtime:instance->string
+            runtime:kind
+            runtime:other-instance+port
+            runtime:other-port
+            runtime:path->instance
+            runtime:port
+            runtime:port*
+            runtime:port-name->instance
+            runtime:runtime-port*))
 
 ;;;
 ;;; Commentary:
@@ -90,8 +84,7 @@
 (define-class <runtime:instance> (<runtime>)
   (ast #:getter .ast #:init-value #f #:init-keyword #:ast) ;; (is? <port) (is? <instance>)
   (container #:getter .container #:init-value #f #:init-keyword #:container) ;;(is? <runtime:instance>)
-  (boundary? #:getter .boundary? #:init-value #f #:init-keyword #:boundary?)
-  (rank #:accessor .rank #:init-value 0 #:init-keyword #:rank)) ; set by runtime:rank!
+  (boundary? #:getter .boundary? #:init-value #f #:init-keyword #:boundary?))
 
 (define-method (runtime:dotted-name (o <runtime:instance>))
   (string-join (runtime:instance->path o) "."))
@@ -128,7 +121,7 @@
   (match o
     (($ <runtime:port>)
      (cond ((eq? o (%sut)) 'interface)
-           ((runtime:requires-instance? o) 'requires)
+           ((ast:requires? (.ast o)) 'requires)
            (else 'provides)))
     (($ <runtime:component>) 'component)
     (($ <runtime:foreign>) 'foreign)
@@ -157,21 +150,22 @@
     (string-join print-path ".")))
 
 (define-method (runtime:instance->path (o <runtime:instance>))
-  (if (runtime:port-instance? (%sut))
-      (if (eq? o (%sut))'("sut") '("client"))
+  (if (is-a? (%sut) <runtime:port>) (if (eq? o (%sut)) '("sut") '("client"))
       (map (compose .name .ast) (reverse (runtime:container-path o)))))
 
 (define-method (runtime:path->instance (o <list>))
-  (or (and (pair? o) (every string? o)) (error (format #f "list of strings expected, got: ~s" o)))
+  (or (and (pair? o) (every string? o))
+      (error (format #f "list of strings expected, got: ~s" o)))
   (match o
     (("sut") (%sut))
     (_
      (let* ((boundary? (= (length o) 1))
-            (container (if boundary? (runtime:find-instance (car o) #:boundary? #t)
-                           (%sut))))
+            (container (if (not boundary?) (%sut)
+                           (runtime:find-instance (car o) #:boundary? #t))))
        (let loop ((container container) (instance-path (cdr o)))
          (if (null? instance-path) container
-             (let ((container (runtime:find-instance (car instance-path) #:container container)))
+             (let ((container (runtime:find-instance (car instance-path)
+                                                     #:container container)))
                (loop container (cdr instance-path)))))))))
 
 (define-method (runtime:port-name->instance (o <string>))
@@ -196,40 +190,6 @@
 (define-method (runtime:container-eq? (a <top>) (b <top>))
   (eq? a b))
 
-(define-method (runtime:port-instance? o)
-  #f)
-
-(define-method (runtime:port-instance? (o <runtime:port>))
-  #t)
-
-(define-method (runtime:provides-instance? (o <runtime:port>))
-  (and (runtime:boundary-port? o)
-       (or (equal? o (%sut))
-           (ast:provides? (.ast o)))))
-
-(define-method (runtime:provides-instance? (o <runtime:instance>))
-  #f)
-
-(define-method (runtime:requires-instance? (o <runtime:port>))
-  (and (runtime:boundary-port? o)
-       (or (equal? o (%sut))
-           (ast:requires? (.ast o)))))
-
-(define-method (runtime:requires-instance? (o <runtime:instance>))
-  #f)
-
-(define-method (runtime:component-instance? (o <runtime:instance>))
-  (is-a? (.type (.ast o)) <component>))
-
-(define-method (runtime:system-instance? (o <runtime:instance>))
-  (is-a? (.type (.ast o)) <system>))
-
-(define-method (runtime:foreign-instance? (o <runtime:instance>))
-  (is-a? (.type (.ast o)) <foreign>))
-
-(define-method (runtime:foreign-instance? (o <boolean>))
-  #f)
-
 
 ;;;
 ;;; Accessors, lookup.
@@ -245,17 +205,7 @@
   '())
 
 (define-method (runtime:port* (o <runtime:instance>))
-  (append (ast:port* (.type (.ast o))) (ast:async-port* (.type (.ast o)))))
-
-(define-method (runtime:other-instance+port (instance <runtime:component>) (port <runtime:port>))
-  (let* ((other-port (runtime:other-port port))
-         (other-instance (if (runtime:boundary-port? other-port) other-port
-                             (.container other-port))))
-    (values other-instance other-port)))
-
-(define-method (runtime:other-instance+port (instance <runtime:port>))
-  (let ((other-port (runtime:other-port instance)))
-    (values (.container other-port) other-port)))
+  (ast:port* (.type (.ast o))))
 
 (define-method (runtime:other-instance+port (instance <runtime:port>) (port <runtime:port>))
   (runtime:other-instance+port instance))
@@ -268,20 +218,12 @@
   (map (cut runtime:find-instance <> #:container o)
        (map .name ((compose ast:port* .type .ast) o))))
 
-(define-method (runtime:runtime-provides-port* (o <runtime:component-model>))
-  (map (cut runtime:find-instance <> #:container o)
-       (map .name ((compose ast:provides-port* .type .ast) o))))
-
-(define-method (runtime:runtime-requires-port* (o <runtime:component-model>))
-  (map (cut runtime:find-instance <> #:container o)
-       (map .name ((compose ast:requires-port* .type .ast) o))))
-
 (define* (runtime:find-instance name #:key container boundary?)
-    (find (lambda (i)
-            (and (runtime:container-eq? (.container i) container)
-                 (equal? (.name (.ast i)) name)
-                 (equal? (.boundary? i) boundary?)))
-          (%instances)))
+  (find (lambda (i)
+          (and (runtime:container-eq? (.container i) container)
+               (equal? (.name (.ast i)) name)
+               (equal? (.boundary? i) boundary?)))
+        (%instances)))
 
 (define-method (ast:sorted-instance* (o <system>))
   (define (end-point-direction? end-point direction instance)
@@ -294,23 +236,30 @@
     (or (end-point-direction? (.left b) direction instance)
         (end-point-direction? (.right b) direction instance)))
   (let ((binding* (ast:binding* o))
-        (instances (map (cut cons <> 0) (ast:instance* o))))
+        (instances (map (cute cons <> 0) (ast:instance* o))))
     (let loop ((all instances) (todo instances) (stable #t))
       (if (null? todo)
-          (if stable (map (cut car <>) (sort all (lambda (a b) (< (cdr a) (cdr b)))))
+          (if stable (map car (sort all (lambda (a b) (< (cdr a) (cdr b)))))
               (loop all all #t))
           (let* ((winst (car todo))
-                 (binding-provides* (filter (lambda (b) (binding-direction? b 'provides (car winst)))
-                                            binding*))
-                 (above* (filter (lambda (wother)
-                                   (find (lambda (b) (binding-direction? b 'requires (car wother)))
-                                         binding-provides*))
-                                 all))
-                 (maxw (if (null? above*) -1 (apply max (map cdr above*))))
+                 (binding-provides*
+                  (filter
+                   (lambda (b)
+                     (binding-direction? b 'provides (car winst)))
+                   binding*))
+                 (above*
+                  (filter
+                   (lambda (wother)
+                     (find (cute binding-direction? <> 'requires (car wother))
+                           binding-provides*))
+                   all))
+                 (maxw (if (null? above*) -1
+                           (apply max (map cdr above*))))
                  (w (max (+ 1 maxw) (cdr winst)))
                  (all (if (= w (cdr winst)) all
-                          (cons (cons (car winst) w) (alist-delete (car winst) all ast:eq?))))
-                 (stable (if (= w (cdr winst)) stable #f)))
+                          (cons (cons (car winst) w)
+                                (alist-delete (car winst) all ast:eq?))))
+                 (stable (and (= w (cdr winst)) stable)))
             (loop all (cdr todo) stable))))))
 
 (define-method (runtime:create-instances (o <runtime:instance>) ast:instance*)
@@ -326,30 +275,40 @@
     (let* ((ast (and=> o .ast))
            (t (and=> ast .type)))
       (match t
-        (($ <interface>) (list o))
-        (($ <component>) (cons o (map (cut port->instance <> o #f) (runtime:port* o))))
+        (($ <interface>)
+         (list o))
+        (($ <component>)
+         (let ((ports (runtime:port* o)))
+           (append
+            (map (cute port->instance <> o #f) (filter ast:provides? ports))
+            (list o)
+            (map (cute port->instance <> o #f) (filter ast:requires? ports)))))
         (($ <foreign>)
          (let* ((ports (runtime:port* o))
-                (port-instances (map (cut port->instance <> o #f) (runtime:port* o)))
+                (port-instances (map (cute port->instance <> o #f) (runtime:port* o)))
                 (inverted-provides (map invert-direction (filter ast:provides? ports)))
                 (inverted-requires (map invert-direction (filter ast:requires? ports)))
-                (provides-instances (map (cut port->instance <> o #t) inverted-requires))
-                (requires-instances (map (cut port->instance <> o #t) inverted-provides)))
-           (cons o (append port-instances provides-instances requires-instances))))
+                (provides-instances (map (cute port->instance <> o #t) inverted-requires))
+                (requires-instances (map (cute port->instance <> o #t) inverted-provides)))
+           (append port-instances provides-instances (list o) requires-instances)))
         (($ <system>)
          (let ((instances (ast:instance* t)))
-           (cons o (append (map (cut port->instance <> o #f) (runtime:port* o))
-                           (append-map (compose model-instances
-                                                (cute ast->runtime:instance <> o))
-                                       instances)))))
+           (let ((ports (runtime:port* o)))
+             (append
+              (map (cute port->instance <> o #f) (filter ast:provides? ports))
+              (list o)
+              (map (cute port->instance <> o #f) (filter ast:requires? ports))
+              (append-map (compose model-instances
+                                   (cute ast->runtime:instance <> o))
+                          instances)))))
         (#f '()))))
 
   (define (injected-instances instances)
     (filter (conjoin (is? <runtime:port>)
-                                   (compose ast:injected? .ast)
-                                   (negate runtime:boundary-port?)
-                                   (negate runtime:other-port))
-                          instances))
+                     (compose ast:injected? .ast)
+                     (negate runtime:boundary-port?)
+                     (negate runtime:other-port))
+            instances))
 
   (define (make-injected-boundary port)
     (make <runtime:port>
@@ -361,7 +320,7 @@
                                   (filter ast:provides? (runtime:port* o))))
          (model-instances (model-instances o))
          (requires-instances (map (cut port->instance <> #f #t)
-                                  (filter (conjoin (negate ast:async?) ast:requires?) (runtime:port* o))))
+                                  (filter ast:requires? (runtime:port* o))))
          (instances (append provides-instances
                             model-instances
                             requires-instances))
@@ -373,8 +332,6 @@
                                        instances)))
          (injected-boundaries (map make-injected-boundary injected-instances))
          (instances (append instances injected-boundaries)))
-    (parameterize ((%instances instances))
-      (for-each (cut runtime:rank! <> 0) (filter runtime:provides-instance? instances)))
     instances))
 
 (define-method (runtime:create-instances (o <runtime:instance>))
@@ -391,31 +348,6 @@
 (define (runtime:%sut-model)
   (runtime:ast-model (%sut)))
 
-
-;;;
-;;; Async, ranking.
-;;;
-
-(define-method (runtime:rank! (o <runtime:port>) r)
-  (let ((other-port (runtime:other-port o)))
-    (when other-port
-      (runtime:rank! (.container other-port) r))))
-
-(define-method (runtime:rank! (o <runtime:component-model>) r)
-  (define (set-rank! port)
-    (let ((r:other-port (runtime:other-port port)))
-      (when r:other-port
-        (and=> (.container r:other-port)
-               (cute runtime:rank! <> (1+ (.rank o)))))))
-  (when (> r (.rank o))
-    (set! (.rank o) r))
-  (when (not (is-a? o <runtime:foreign>))
-    (for-each set-rank! (filter (compose not ast:injected? .ast)
-                                (runtime:runtime-requires-port* o)))))
-
-(define-method (runtime:rank! (o <boolean>) r)
-  *unspecified*)
-
 
 ;;;
 ;;; Plumbing
@@ -463,7 +395,7 @@
 
         (define (inner-runtime-port o)
           (if (and (not (runtime:boundary-port? o))
-                   (runtime:foreign-instance? (.container o)))
+                   (is-a? (.container o) <runtime:foreign>))
               (runtime:find-instance (.name (.ast o)) #:container (.container o) #:boundary? #t)
               (let ((other (ast:other-end-point (.ast o))))
                 (and other
@@ -492,14 +424,16 @@
                                        #:boundary? (not (.boundary? o))))))
 
         (define (runtime:system-or-foreign-instance? c)
-          (and c (or (runtime:system-instance? c) (runtime:foreign-instance? c))))
+          (and c
+               (or (is-a? c <runtime:system>)
+                   (is-a? c <runtime:foreign>))))
 
         (cond ((runtime:boundary-port? o)
                (let ((next (runtime:find-instance (.name (.ast o)) #:container (or container sut))))
                  (cond ((not next) o)
-                       ((runtime:component-instance? (.container next)) next)
+                       ((is-a? (.container next) <runtime:component>) next)
                        (else (loop next o)))))
-              ((runtime:system-instance? container)
+              ((is-a? container <runtime:system>)
                (let* ((inner (inner-runtime-port o))
                       (outer (outer-runtime-port o))
                       (next (if (eq? inner previous) outer inner)))
@@ -509,7 +443,7 @@
                         (loop next o))
                        ((not outer) (injected-port o))
                        (else next))))
-              ((runtime:foreign-instance? container)
+              ((is-a? container <runtime:foreign>)
                (let* ((inner (inner-runtime-port o))
                       (outer (outer-runtime-port o))
                       (next (if (eq? inner previous) outer inner)))
@@ -517,12 +451,21 @@
                        ((runtime:boundary-port? next) next)
                        ((runtime:system-or-foreign-instance? (.container next)) (loop next o))
                        (else next))))
-              ((runtime:component-instance? container)
+              ((is-a? container <runtime:component>)
                (let ((next (outer-runtime-port o)))
                  (cond
                   ((not next) (injected-port o))
                   ((runtime:system-or-foreign-instance? (.container next)) (loop next o))
                   (else next)))))))))
 
-(define (runtime:other-port o) (runtime:other-port-unmemoized o))
-(define runtime:other-port (pure-funcq runtime:other-port))
+(define runtime:other-port runtime:other-port-unmemoized)
+
+(define-method (runtime:other-instance+port (instance <runtime:component>) (port <runtime:port>))
+  (let* ((other-port (runtime:other-port port))
+         (other-instance (if (runtime:boundary-port? other-port) other-port
+                             (.container other-port))))
+    (values other-instance other-port)))
+
+(define-method (runtime:other-instance+port (instance <runtime:port>))
+  (let ((other-port (runtime:other-port instance)))
+    (values (.container other-port) other-port)))
