@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2019, 2020, 2021, 2022, 2023, 2024, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018, 2019, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2021, 2022 Paul Hoogendijk <paul@dezyne.org>
@@ -31,7 +31,7 @@
   #:use-module (ice-9 regex)
   #:use-module (ice-9 string-fun)
 
-  #:use-module (dzn ast goops)
+  #:use-module (dzn ast ast)
   #:use-module (dzn ast)
   #:use-module (dzn config)
   #:use-module (dzn misc)
@@ -41,7 +41,6 @@
   #:use-module (dzn vm runtime)
   #:export (%%debug
             %debug
-            %debug?
             %next-input
             %startup-info
             %strict?
@@ -151,9 +150,6 @@
 ;;;
 ;;; Code:
 
-;;; Should debug info be printed?
-(define %debug? (make-parameter #f))
-
 ;; Is the input trail to be matched exactly?
 (define %strict? (make-parameter #f))
 
@@ -442,9 +438,24 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
     (let ((ports (filter
                   (conjoin runtime:boundary-port?
                            runtime:other-port
-                           (negate (compose (is? <runtime:foreign>)
-                                            .container
-                                            runtime:other-port)))
+                           (disjoin
+                            (negate (compose (is? <runtime:foreign>)
+                                             .container
+                                             runtime:other-port))
+                            ;; when a port belongs to a foreign component
+                            ;; it is like a boundary port when it is
+                            ;; connected to another components port
+                            ;; in the complementary direction (provides/requires)
+                            (conjoin ast:provides?
+                                     (compose runtime:boundary-port?
+                                              runtime:other-port)
+                                     (compose ast:requires?
+                                              runtime:other-port))
+                            (conjoin ast:requires?
+                                     (compose runtime:boundary-port?
+                                              runtime:other-port)
+                                     (compose ast:provides?
+                                              runtime:other-port))))
                   (%instances))))
       (append-map
        (lambda (p)
@@ -1104,11 +1115,14 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (define (assign state variable value)
     (let ((name (.name variable)))
       (or (range-error variable value)
-          (clone state #:variables (assoc-set! (copy-tree (.variables state)) name value)))))
+          (let* ((variables (.variables state))
+                 (variables (assoc-set! (copy-tree variables) name value)))
+            (clone state #:variables variables)))))
 
   (define (update-variable update-list variable state)
     (let ((name (string->symbol (.name variable))))
-      (or (and=> (assoc-ref update-list name) (compose (cut assign state variable <>) sexp->value))
+      (or (and=> (assoc-ref update-list name)
+                 (compose (cute assign state variable <>) sexp->value))
           state)))
 
   (define (update-state event state pc)
@@ -1231,7 +1245,8 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (let* ((name (.name variable))
          (state (get-state pc))
          (value (eval-expression pc expression))
-         (state (clone state #:variables (assoc-set! (copy-tree (.variables state)) name value)))
+         (variables (assoc-set! (copy-tree (.variables state)) name value))
+         (state (clone state #:variables variables))
          (pc (set-state pc state))
          (error (range-error variable value)))
     (if (not error) pc
@@ -1294,18 +1309,17 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
   (define (variables-prefixed-with-name port)
     (map (match-lambda
            ((variable . value)
-            (cons (string-append ((compose .name .ast .instance) port)
-                                 "."
-                                 variable) value)))
+            (let* ((prefix ((compose .name .ast .instance) port))
+                   (name (string-append prefix "." variable)))
+              `(,name . ,value))))
          (.variables port)))
 
-  (let ((ports (filter (compose (cute eq? (.instance component) <>)
-                                .container .instance)
-                       state-list)))
-    (clone component
-           #:variables (append
-                        (append-map variables-prefixed-with-name ports)
-                        (.variables component)))))
+  (let* ((ports (filter (compose (cute eq? (.instance component) <>)
+                                 .container .instance)
+                        state-list))
+         (port-variables (append-map variables-prefixed-with-name ports))
+         (variables (append port-variables (.variables component))))
+    (clone component #:variables variables)))
 
 (define-method (serialize (o <system-state>) port)
   (let* ((state-list (.state-list o))
@@ -1744,6 +1758,10 @@ See <https://www.gnu.org/licenses/agpl.html>, for more details.
                  (clone pc #:status (car errors)))))
     pc))
 
-(define-method (make-implicit-illegal (pc <program-counter>) (o <ast>))
-  (let ((illegal (make <implicit-illegal-error> #:ast o #:message "illegal")))
-    (clone pc #:previous #f #:status illegal)))
+(define-method (make-implicit-illegal (pc <program-counter>) (o <initial-compound>))
+  (let* ((location (.location o))
+         (illegal (make <declarative-illegal> #:location location))
+         (trigger (.trigger pc))
+         (triggers (make <triggers> #:elements (list trigger)))
+         (on (make <on> #:statement illegal #:triggers triggers #:location location)))
+    (clone on #:parent o)))
