@@ -1,6 +1,6 @@
 // dzn-runtime -- Dezyne runtime library
 //
-// Copyright © 2016, 2019 Jan Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2016, 2019, 2023 Jan Nieuwenhuizen <janneke@gnu.org>
 // Copyright © 2016 Rob Wieringa <rob@dezyne.org>
 // Copyright © 2018 Filip Toman <filip.toman@verum.com>
 // Copyright © 2016 Rutger van Beusekom <rutger@dezyne.org>
@@ -26,11 +26,14 @@
 
 #ifndef DZN_MEM_H
 #define DZN_MEM_H
+
 #include <dzn/config.h>
-#include <dzn/boolc90.h>
+
+#include <stdbool.h>
 #include <stddef.h>
-void  dzn_free(void* ptr);
-void* dzn_calloc (size_t n, size_t size);
-void* dzn_malloc (size_t size);
+
+void  dzn_free (void *ptr);
+void *dzn_calloc (size_t n, size_t size);
+void *dzn_malloc (size_t size);
 
 #endif /* DZN_MEM_H */
