@@ -1,7 +1,7 @@
 // Dezyne --- Dezyne command line tools
 //
 // Copyright © 2018 Filip Toman <filip.toman@verum.com>
-// Copyright © 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+// Copyright © 2021, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 //
 // This file is part of Dezyne.
 //
@@ -25,23 +25,25 @@
 #include <foreign_reply.h>
 
 /* call "name-of-foreign"_skel_init as follows:
-   foreign_skel_init(&(self->base), dezyne_locator, dzn_meta); */
+   foreign_skel_init(&(self->base), dzn_locator, dzn_meta); */
 
-void Foreign_init(Foreign* self, locator* dezyne_locator
-#if DZN_TRACING
-, dzn_meta* dzn_meta
+void
+Foreign_init (Foreign *self, dzn_locator *dzn_locator
+#if 1 //DZN_TRACING
+              , dzn_meta *dzn_meta
 #endif /* !DZN_TRACING */
-)
+             )
 {
-  Foreign_skel_init(&(self->base), dezyne_locator
-#if DZN_TRACING
-, dzn_meta
+  Foreign_skel_init (& (self->base), dzn_locator
+#if 1 //DZN_TRACING
+                     , dzn_meta
 #endif /* !DZN_TRACING */
-);
+                    );
 }
 
-bool Foreign_w_world(Foreign* self)
+bool
+Foreign_w_world (Foreign *self)
 {
-  self->reply__bool = true;
-  return self->reply__bool;
+  self->dzn_reply_bool = true;
+  return self->dzn_reply_bool;
 }
