@@ -1,8 +1,8 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2020 Johri van Eerd <vaneerd.johri@gmail.com>
-;;; Copyright © 2020, 2021, 2022, 2023 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2020, 2021, 2022, 2023, 2025 Janneke Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2020, 2021, 2025 Rutger (regtur) van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -38,12 +38,13 @@
   #:use-module (ice-9 rdelim)
 
   #:use-module (dzn commands language)
+  #:use-module (dzn misc)
   #:use-module (dzn parse)
   #:use-module (dzn parse peg)
   #:use-module (dzn parse complete)
+  #:use-module (dzn parse location)
   #:use-module (dzn parse lookup)
   #:use-module (dzn parse tree)
-  #:use-module (dzn parse util)
   #:use-module (dzn shell-util)
   #:use-module (test dzn automake))
 
@@ -58,13 +59,11 @@
 
 (define (file-name->parse-tree file-name)
   (let ((text (file-name->text file-name)))
-    (parameterize ((%peg:fall-back? #t))
-      (string->parse-tree text #:file-name file-name))))
+    (parse:string->tree* text #:file-name file-name)))
 
 (define* (test-context #:key file-name text line (column 0) offset)
   (let* ((text   (or text (file-name->text file-name)))
-         (root   (parameterize ((%peg:fall-back? #t))
-                   (string->parse-tree text #:file-name file-name)))
+         (root   (parse:string->tree* text #:file-name file-name))
          (offset (or offset
                      (and line (line-column->offset line column text))
                      (string-length text))))
@@ -366,21 +365,17 @@
   '("blocking" "external" "injected" "ihello")
   (test-complete #:file-name "component-requires.dzn" #:line 14 #:column 11))
 
-(test-equal "component-requires external"
-  '("blocking" "injected" "ihello")
+(test-equal "component-requires blocking"
+  '("ihello")
   (test-complete #:file-name "component-requires.dzn" #:line 15 #:column 20))
 
-(test-equal "component-requires injected"
-  '("blocking" "external" "ihello")
+(test-equal "component-requires external"
+  '("ihello")
   (test-complete #:file-name "component-requires.dzn" #:line 16 #:column 20))
 
-(test-equal "component-requires external injected"
-  '("blocking" "ihello")
-  (test-complete #:file-name "component-requires.dzn" #:line 17 #:column 29))
-
-(test-equal "component-requires injected external"
-  '("blocking" "ihello")
-  (test-complete #:file-name "component-requires.dzn" #:line 18 #:column 29))
+(test-equal "component-requires injected"
+  '("ihello")
+  (test-complete #:file-name "component-requires.dzn" #:line 17 #:column 20))
 
 (test-equal "component-empty"
   '("provides" "requires")
@@ -505,11 +500,11 @@
 
 (test-equal "field-test field"
   '("e.False" "e.True")
-  (test-complete #:file-name "enum.dzn" #:line 9 #:column 7))
+  (test-complete #:file-name "enum.dzn" #:line 16 #:column 7))
 
 (test-equal "enum-literal with comment"
   '("Tri_bool.False" "Tri_bool.True" "Tri_bool.Whatever")
-  (test-complete #:file-name "enum.dzn" #:line 10 #:column 17))
+  (test-complete #:file-name "enum.dzn" #:line 17 #:column 17))
 
 (test-equal "import requires"
   '("blocking" "external" "injected" "ihello" "ihello_enum" "ihello_int")
@@ -610,11 +605,11 @@
 
 (test-equal "system binding instance provides"
   '("d.ww" "h")
-  (test-complete #:file-name "system-binding.dzn" #:line 52 #:column 12))
+  (test-complete #:file-name "system-binding.dzn" #:line 52 #:column 14))
 
 (test-equal "system binding instance requires"
   '("d.hh" "w")
-  (test-complete #:file-name "system-binding.dzn" #:line 53 #:column 12))
+  (test-complete #:file-name "system-binding.dzn" #:line 53 #:column 14))
 
 (test-equal "system end-point provides instance."
   '("c.hh" "d.hh")
@@ -772,8 +767,8 @@
 (test-begin "lookup")
 
 (test-equal "port->interface"
-  "lookup.dzn:56:10"
-  (test-lookup #:file-name "lookup.dzn" #:line 46 #:column 11))
+  "lookup.dzn:43:10"
+  (test-lookup #:file-name "lookup.dzn" #:line 60 #:column 11))
 
 (test-equal "instance->component"
   "lookup.dzn:26:10"
@@ -797,8 +792,8 @@
   (test-lookup #:file-name "lookup.dzn" #:line 34 #:column 4))
 
 (test-equal "variable-expression->variable"
-  "lookup.dzn:50:9"
-  (test-lookup #:file-name "lookup.dzn" #:line 52 #:column 54))
+  "lookup.dzn:64:9"
+  (test-lookup #:file-name "lookup.dzn" #:line 66 #:column 54))
 
 (test-equal "enum-literal->type"
   "lookup.dzn:33:9"
@@ -813,16 +808,16 @@
   (test-lookup #:file-name "ilookup.dzn" #:line 32 #:column 7))
 
 (test-equal "trigger->port"
-  "lookup.dzn:46:25"
-  (test-lookup #:file-name "lookup.dzn" #:line 52 #:column 12))
+  "lookup.dzn:60:25"
+  (test-lookup #:file-name "lookup.dzn" #:line 66 #:column 12))
 
 (test-equal "action->port"
-  "lookup.dzn:46:25"
-  (test-lookup #:file-name "lookup.dzn" #:line 51 #:column 24))
+  "lookup.dzn:60:25"
+  (test-lookup #:file-name "lookup.dzn" #:line 65 #:column 24))
 
 (test-equal "action->event"
-  "lookup.dzn:58:10"
-  (test-lookup #:file-name "lookup.dzn" #:line 51 #:column 27))
+  "lookup.dzn:45:10"
+  (test-lookup #:file-name "lookup.dzn" #:line 65 #:column 27))
 
 (test-equal "end-point->port"
   "lookup.dzn:72:18"
@@ -877,17 +872,17 @@
   "int.dzn:1:7"
   (test-lookup #:file-name "int.dzn" #:line 8 #:column 4))
 
-(test-equal "field-test->var"
-  "enum.dzn:8:9"
-  (test-lookup #:file-name "enum.dzn" #:line 9 #:column 5))
+(test-equal "field-test->reference"
+  "enum.dzn:15:9"
+  (test-lookup #:file-name "enum.dzn" #:line 16 #:column 5))
 
 (test-equal "field-test->enum-field"
   "enum.dzn:1:17"
-  (test-lookup #:file-name "enum.dzn" #:line 9 #:column 7))
+  (test-lookup #:file-name "enum.dzn" #:line 16 #:column 7))
 
 (test-equal "enum-literal->field with comment"
-  "enum.dzn:18:2"
-  (test-lookup #:file-name "enum.dzn" #:line 10 #:column 26))
+  "enum.dzn:7:2"
+  (test-lookup #:file-name "enum.dzn" #:line 17 #:column 26))
 
 (test-equal "variable-type->interface enum"
   "interface-enum.dzn:3:7"
